"""
TBD
"""

__author__ = "Lukas Mahler"
__version__ = "0.0.0"
__date__ = "05.05.2022"
__email__ = "m@hler.eu"
__status__ = "Development"


# Default
import sys
import time
import shutil
import inspect
import logging
import pathlib
from logging.handlers import RotatingFileHandler

# Custom
import toml


def getConf(fname, log):
    """

    """
    p = pathlib.Path(fname)

    if p.suffix == ".toml":
        if p.is_file() and p.exists():
            try:
                config = toml.load(fname)
                if isValidConf(config):
                    return config
                else:
                    log.pipeOut(f"The provided '.toml' is invalid", lvl="critical")
            except ValueError as e:
                log.pipeOut(f"The provided '.toml' is probably invalid, returned error: [{e}]", lvl="critical")
        else:
            log.pipeOut(f"Couldn't locate the '.toml' file [{fname}].", lvl="error")
            log.pipeOut("Creating a new '.toml' file from template, please edit and restart.")
            shutil.copy("src/template.toml", fname)
            exit(1)
    else:
        log.pipeOut(f"The provided config file [{fname}] is not a '.toml' file.", lvl="error")
        log.pipeOut("Creating a new '.toml' file from template, please edit and restart.")
        shutil.copy("src/template.toml", "prod.toml")
        exit(1)


def isValidConf(config):
    """

    """
    valid = True

    required = {'Auth': ['token', 'server'], 'Swap': ['apikey'], 'Log': ['level']}

    for required_header in required:
        if required_header in config:
            for required_key in required[required_header]:
                if required_key not in config[required_header]:
                    print(f"[Err] Missing key {required_key}")
                    valid = False
                else:
                    if config[required_header][required_key] == "":
                        print(f"[Err] Key {required_key} can't be empty")
                        valid = False
        else:
            print(f"[Err] Missing header {required_header}")
            valid = False

    return valid


class Logger:
    """
    Create a rotating log in a log folder
    """

    def __init__(self, logpath, logname, reprint=True, lvl="INFO"):

        self.reprint = reprint
        self.log = logging.getLogger("mylog")
        p = pathlib.Path(logpath)
        p.mkdir(parents=True, exist_ok=True)

        handler = RotatingFileHandler(pathlib.Path(logpath, logname),
                                      encoding='utf-8',
                                      maxBytes=1 * 1024 * 1024,
                                      backupCount=10)
        logformat = logging.Formatter("%(asctime)s %(levelname)8s %(message)s", "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(logformat)
        self.log.addHandler(handler)
        self.log.setLevel(lvl)

    def changeLevel(self, lvl):
        """
        Change the loglevel after creation
        """
        self.log.setLevel(logging.getLevelName(lvl))

    def pipeOut(self, msg, lvl="INFO"):
        """
        All output pipes through this function.
        It's possible to print the output to console [set reprint to True]
        or run in silent log mode. [set reprint to False]
        """

        # caller = inspect.currentframe().f_back.f_code.co_name  # Returns caller function name
        caller = pathlib.Path(inspect.stack()[1].filename).stem  # Returns caller filename without extension

        lvl = lvl.upper()
        lvln = int(getattr(logging, lvl))
        self.log.log(lvln, msg)

        if self.reprint and lvln >= self.log.getEffectiveLevel():
            print(f"[{time.strftime('%H:%M:%S')}]{f'[{lvl}]':10s}[{caller}] {msg}")

        if lvl == "CRITICAL":
            exit(1)

    def unhandledException(self, exc_type, exc_value, exc_traceback):
        """
        src = https://stackoverflow.com/a/16993115/5593051
        """

        if issubclass(exc_type, KeyboardInterrupt):
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
            return

        self.log.critical("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))


if __name__ == "__main__":
    exit()
